import logging
import sys

if sys.argv[0].find('client') == -1:
    SERVER_LOGGER = logging.getLogger('server')
else:
    SERVER_LOGGER = logging.getLogger('client')


class Port:
    """
    Класс - дескриптор для номера порта.
    Позволяет использовать только порты с 1023 по 65536.
    При попытке установить неподходящий номер порта генерирует исключение.
    """
    def __set__(self, instance, value):
        if not 1023 < value < 65536:
            SERVER_LOGGER.critical(f'Попытка запуска сервера с указанием неподходящего порта {value}. Допустимы '
                                   f'адреса с 1024 до 65535.')
            sys.exit(1)
        instance.__dict__[self.name] = value

    def __set_name__(self, owner, name):
        self.name = name
